import logging

from docutils import nodes
from docutils.parsers.rst import directives, Directive

logger = logging.getLogger(__name__)


class Pygments(Directive):
    """ Source code syntax hightlighting.

    Example:

        .. sourcecode:: python

            My code goes here.

    If you want line numbers, use it like this:

        .. sourcecode:: python
            :linenos:

            My code goes here.

    If you want to use it, don't forget to generate style css.
    """
    required_arguments = 1
    optional_arguments = 0
    final_argument_whitespace = True
    has_content = True

    def __init__(self, *args, **kwargs):
        self.VARIANTS = {
            'default': HtmlFormatter(noclasses=False),
            'linenos': HtmlFormatter(noclasses=False, linenos=True),
            }
        self.option_spec = dict((key, directives.flag) for key in self.VARIANTS)
        super(Pygments, self).__init__(*args, **kwargs)

    def run(self):
        self.assert_has_content()
        try:
            lexer = get_lexer_by_name(self.arguments[0])
        except ValueError:
            # no lexer found - use the text one instead of an exception
            lexer = TextLexer()
        # take an arbitrary option if more than one is given
        variant = self.options and self.options.keys()[0] or 'default'
        formatter = self.VARIANTS[variant]
        parsed = highlight(u'\n'.join(self.content), lexer, formatter)
        return [nodes.raw('', parsed, format='html')]

try:
    from pygments import highlight
    from pygments.lexers import get_lexer_by_name, TextLexer
    from pygments.formatters import HtmlFormatter
    directives.register_directive('sourcecode', Pygments)
except ImportError:
    logger.debug('Pygments are not installed')



'''Parser of naive data format
Copyright 2009 Alexander Solovyov, under terms of Poetic License

Format:

  key: value
  key: [list, of, values]
  key: {key: value, key: value}
  key: date: yyyy-mm-dd HH:MM:SS
  key: True  # boolean value
  key: False # boolean value

Any line without a ":" is simply skipped. Field with a key of ``date`` are
special case and are checked to contain date value.
'''

from datetime import datetime


def parse(data):
    result = {}
    for line in data.splitlines():
        if ':' in line:
            key, value = parse_line(line)
            result[key] = value
    return result


def parse_date(date):
    return datetime.strptime(date, '%Y-%m-%d %H:%M:%S')


def parse_line(line):
    key, value = strip(line.split(':', 1))
    s, e = value.startswith, value.endswith
    if s('[') and e(']'):
        value = strip(value[1:-1].split(','))
    elif s('{') and e('}'):
        value = dict(strip(x.split(':')) for x in value[1:-1].split(','))
    elif s('date:'):
        value = parse_date(value[len('date:'):].strip())
    elif key.strip() == 'date':
        try:
            value = parse_date(value)
        except ValueError:
            pass
    elif value.lower() in 'true yes on'.split():
        value = True
    elif value.lower() in 'false no off'.split():
        value = False
    return key, value


def strip(lst):
    return [x.strip() for x in lst]


class Settings(dict):
    def __init__(self, parent=None, **kwargs):
        self.parent = parent
        super(Settings, self).__init__(**kwargs)

    def read(self, inp):
        self.update(parse(inp))
        return self

    def __getitem__(self, name):
        try:
            return super(Settings, self).__getitem__(name)
        except KeyError:
            if self.parent:
                return self.parent[name]
            raise

    def __getattr__(self, name):
        try:
            return self[name]
        except KeyError, e:
            raise AttributeError(str(e))

    def __setattr__(self, name, value):
        self[name] = value



import os
import posixpath
from SimpleHTTPServer import SimpleHTTPRequestHandler
from BaseHTTPServer import HTTPServer
import urllib
import logging

from cyrax import autoreload, core

logger = logging.getLogger(__name__)


class CyraxHTTPRequestHandler(SimpleHTTPRequestHandler):

    def translate_path(self, path):
        """
        The default behavior of SimpleHTTPRequestHandler.translate_path
        is to serve files from the current directory and all of the
        directories below it. This class overrides the method and
        looks for a `rootpath` attribute, which will be used instead
        of the current directory.
        """
        # abandon query parameters
        path = path.split('?', 1)[0]
        path = path.split('#', 1)[0]
        path = posixpath.normpath(urllib.unquote(path))
        words = path.split('/')
        words = filter(None, words)
        try:
            path = self.rootpath
        except AttributeError:
            path = os.getcwd()
        for word in words:
            drive, word = os.path.splitdrive(word)
            head, word = os.path.split(word)
            if word in (os.curdir, os.pardir):
                continue
            path = os.path.join(path, word)
        return path

def run_server(address, port, source, dest):
    core.Site(source, dest).render()
    CyraxHTTPRequestHandler.rootpath = dest
    httpd = HTTPServer((address, port), CyraxHTTPRequestHandler)
    logger.info("Serving at http://%s:%s", address, port)
    try:
        httpd.serve_forever()
    except KeyboardInterrupt:
        sys.exit(0)

def start_server(address, port, source, dest):
    autoreload.main(run_server, (), {
        'address': address,
        'port': port,
        'source': source,
        'dest': dest,
    })



from jinja2 import Environment, FileSystemLoader, Undefined, ext

from cyrax.template import (typogrify, templatefilters, templatetags,
                            templatefunctions)


class LoyalUndefined(Undefined):
    def __getattr__(self, name):
        return self

    __getitem__ = __getattr__

    def __call__(self, *args, **kwargs):
        return self


def initialize_env(source):
    '''
    Initialize environment.
    '''
    loader = FileSystemLoader(source)

    env = Environment(loader=loader, undefined=LoyalUndefined,
                      extensions=[templatetags.MetaInfoExtension,
                                  templatetags.MarkExtension,
                                  ext.with_])

    filters = dict((f.__name__, f) for f in typogrify.filters)
    env.filters.update(filters)
    filters = dict((f.__name__, f) for f in templatefilters.filters)
    env.filters.update(filters)
    functions = dict((f.__name__, f) for f in templatefunctions.functions)
    env.globals.update(functions)
    return env



from jinja2 import contextfunction

from cyrax import utils


@contextfunction
def url_for(context, path):
    if not path:
        return ''
    basepath = utils.base_path(context['site'].url)
    return utils.relpath(
        utils.safe_url_join(basepath, context['entry'].get_relative_url()),
        utils.safe_url_join(basepath, path))


@contextfunction
def absolute_url_for(context, path):
    return utils.safe_url_join(context['site'].url, path)


functions = [url_for, absolute_url_for]




# Autoreloading launcher.
# Borrowed from Peter Hunt and the CherryPy project (http://www.cherrypy.org).
# Some taken from Ian Bicking's Paste (http://pythonpaste.org/).
#
# Borrowed from Django and adapted for Cyrax.
#
# Portions copyright (c) 2004, CherryPy Team (team@cherrypy.org)
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without modification,
# are permitted provided that the following conditions are met:
#
#     * Redistributions of source code must retain the above copyright notice,
#       this list of conditions and the following disclaimer.
#     * Redistributions in binary form must reproduce the above copyright notice,
#       this list of conditions and the following disclaimer in the documentation
#       and/or other materials provided with the distribution.
#     * Neither the name of the CherryPy Team nor the names of its contributors
#       may be used to endorse or promote products derived from this software
#       without specific prior written permission.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
# ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE LIABLE
# FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
# DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
# CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
# OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import os, sys, time

import logging

try:
    import thread
except ImportError:
    import dummy_thread as thread

# This import does nothing, but it's necessary to avoid some race conditions
# in the threading module. See http://code.djangoproject.com/ticket/2330 .
try:
    import threading
except ImportError:
    pass

logger = logging.getLogger(__name__)

RUN_RELOADER = True

_mtimes = {}
_win = (sys.platform == "win32")


def normpath(p):
    return os.path.normpath(os.path.abspath(p))


def _should_ignore(path, exclude):
    root, subpart = os.path.split(path)
    if root == path:
        # reached the /
        return False

    for exc in exclude:
        if normpath(path) == normpath(exc):
            return True

    return _should_ignore(root, exclude)


def _get_mtime(path):
    try:
        stat = os.stat(path)
    except OSError:
        return None
    mtime = stat.st_mtime
    if _win:
        mtime -= stat.st_ctime
    return mtime


def source_changed(path, exclude=None):
    global _mtimes
    if exclude is None:
        exclude = []
    for root, dirs, files in os.walk(path):
        if _should_ignore(root, exclude):
            continue
        for filename in files:
            filepath = os.path.join(root, filename)
            mtime = _get_mtime(filepath)
            if filepath not in _mtimes:
                _mtimes[filepath] = mtime
                continue
            if mtime != _mtimes[filepath]:
                logger.debug('File %r is changed', filepath)
                _mtimes = {}
                return True
    return False


def reloader_thread(source, dest):
    while RUN_RELOADER:
        if source_changed(source, exclude=[dest]):
            logger.info('Source changed, restarting')
            sys.exit(3) # force reload
        time.sleep(1)


def restart_with_reloader():
    while True:
        args = [sys.executable] + sys.argv
        if sys.platform == "win32":
            args = ['"%s"' % arg for arg in args]
        new_environ = os.environ.copy()
        new_environ["RUN_MAIN"] = 'true'
        exit_code = os.spawnve(os.P_WAIT, sys.executable, args, new_environ)
        if exit_code != 3:
            return exit_code


def reloader(main_func, args, kwargs):
    if os.environ.get("RUN_MAIN") == "true":
        thread.start_new_thread(main_func, args, kwargs)
        try:
            reloader_thread(kwargs['source'], kwargs['dest'])
        except KeyboardInterrupt:
            pass
    else:
        try:
            sys.exit(restart_with_reloader())
        except KeyboardInterrupt:
            pass


def main(main_func, args=None, kwargs=None):
    if args is None:
        args = ()
    if kwargs is None:
        kwargs = {}
    reloader(main_func, args, kwargs)




import sys, logging
import os.path as op
from optparse import OptionParser

from cyrax.core import Site
from cyrax.server import start_server

__version__ = '2.1'
logger = logging.getLogger('cyrax')

def main():
    usage = '%s [options] [source]' % sys.argv[0]
    parser = OptionParser(usage)

    parser.add_option('', '--log', default='-',
                      help='redirect log to this file (default: stdout)')
    parser.add_option('-q', '--quiet', default=False, action='store_true',
                      help='do not output information messages')
    parser.add_option('-v', '--verbose', default=False, action='store_true',
                      help='output more logs')

    parser.add_option('-w', '--webserve', default=False, action='store_true',
                      help='start local web server')
    parser.add_option('-a', '--address', default='127.0.0.1',
                      help='address to listen on (default: all interfaces)')
    parser.add_option('-p', '--port', default=8000, type='int',
                      help='port to listen on (default: 8000)')

    parser.add_option('-d', '--dest', default='',
        help='destination directory (default: _build in source dir)')

    opts, args = parser.parse_args()

    if opts.verbose:
        level = logging.DEBUG
    elif opts.quiet:
        level = logging.ERROR
    else:
        level = logging.INFO

    if opts.log == '-':
        logging.basicConfig(stream=sys.stdout, level=level)
    else:
        logging.basicConfig(filename=opts.log, level=level)

    source = op.abspath(args and args[0] or '.')
    if not op.exists(op.join(source, 'settings.cfg')):
        logger.error("Can't find settings.cfg in the current folder")
        sys.exit(1)

    dest = opts.dest and opts.dest or op.join(source, '_build')

    if opts.webserve:
        start_server(opts.address, opts.port, source, dest)
    else:
        site = Site(source, dest)
        site.render()



import re
import time
import datetime
import posixpath
import os.path as op
import logging

from cyrax.conf import Settings
from cyrax.events import events
from cyrax.utils import (safe_url_join, url2path, path2url,
                         base_path, makedirs)


logger = logging.getLogger(__name__)
DATE_RE = re.compile(r'(.*?)(\d+)[/-](\d+)[/-](\d+)[/-](.*)$')


def postcmp(x, y):
    xd = x.settings.get('updated', x.settings.date)
    yd = y.settings.get('updated', y.settings.date)
    return -1 if xd < yd else 1 if xd > yd else 0


class Entry(object):
    def __init__(self, site, path, source=None):
        '''Initialize an entry

        This involves change of base class by running static method Class.check
        of every member of models.TYPE_LIST.

        Arguments:

         - `site`: site this entry belongs to
         - `path`: relative path to source template and to result
         - `source`: optional source template path. Can be used to trick
            system to have virtual entries (with no real equivalent in
            source directory)
        '''
        self.site = site
        self.path = path
        self.source = source
        self.mtime = self.get_mtime()

        self.settings = Settings(parent=self.site.settings)

        base = '_%s.html' % self.__class__.__name__.lower()
        if op.exists(op.join(self.site.root, base)):
            self.settings.parent_tmpl = base

        self.template = self.get_template()
        self.settings.base, self.settings.slug = op.split(self.path)
        self.collect()
        if hasattr(self, 'init'):
            self.init()

    def __repr__(self):
        return '<%s: %r>' % (self.__class__.__name__, self.path)

    def __getitem__(self, name):
        return self.settings[name]

    def __getattr__(self, name):
        try:
            return self.settings[name]
        except KeyError:
            raise AttributeError(name)

    def get_mtime(self):
        '''Determine modification time (with date)

        As in modification time vs creation time

        Thinks that if source is not null, then entry is virtual
        and returns current datetime
        '''
        if self.source:
            return datetime.datetime.now()
        mtime = op.getmtime(op.join(self.site.root, self.path))
        return datetime.datetime(*time.gmtime(mtime)[:6])

    def get_template(self):
        '''Get Jinja2 template of entry to render
        '''
        return self.site.env.get_template(self.source or self.path,
                                          globals={'entry': self})

    def collect(self):
        '''Collect settings information from entry

        Renders Jinja2 template to get this information from {% meta %}
        '''
        self.template.render()

    def isdir(self):
        '''Determines if entry should be rendered as directory
        '''
        return self.settings.get('isdir', True)

    def get_dest(self):
        path = op.join(self.site.dest, url2path(self.get_relative_url()))
        if self.isdir():
            path = op.join(path, 'index.html')
        return path

    def get_relative_url(self):
        return self.path

    def get_url(self):
        return safe_url_join(base_path(self.site.url), self.get_relative_url())

    def get_absolute_url(self):
        return safe_url_join(self.site.url, self.get_relative_url())

    def render(self):
        logger.info('Rendering %r' % self)
        # workaround for a dumb bug
        # no ideas why but all tag templates contain same self inside
        self.template.globals['entry'] = self
        path = self.get_dest()
        makedirs(op.dirname(path))
        file(path, 'w').write(self.template.render().encode('utf-8'))


class NonHTML(Entry):
    @staticmethod
    def check(site, path):
        return not path.endswith('.html')

    def isdir(self):
        return self.settings.get('isdir', False)


class Post(Entry):
    @staticmethod
    def check(site, path):
        if DATE_RE.search(path):
            return True
        return False

    def __init__(self, site, path, source=None):
        base, Y, M, D, slug = DATE_RE.search(path).groups()
        self.date = datetime.datetime(int(Y), int(M), int(D))
        super(Post, self).__init__(site, path, source)
        self.settings.base = base
        self.settings.slug = op.splitext(slug)[0]

    def init(self):
        # dumb hack
        self.settings.date = self.date
        self.site.posts.append(self)
        self.site.posts.sort(cmp=postcmp, reverse=True)
        self.site.latest_post = self.site.posts[0]

        self._process_tags()

    def _process_tags(self):
        for tag in self.settings.get('tags', []):
            tagentries = self.site.tags.setdefault(tag, [])
            if self not in tagentries:
                tagentries.append(self)
                self.site.tags[tag].sort(cmp=postcmp, reverse=True)

    def __str__(self):
        return self.slug

    def get_relative_url(self):
        date = self.date.strftime('%Y/%m/%d')
        url = posixpath.join(self.base, date, self.slug) + '/'
        return url

    @staticmethod
    def register(site):
        site.posts = []

events.connect('traverse-started', Post.register)


class Page(Entry):
    @staticmethod
    def check(site, path):
        return path.endswith('.html')

    def init(self):
        if self.isdir():
            if self.path.endswith('index.html'):
                path = self.path[:-len('index.html')]
            else:
                path = op.splitext(self.path)[0]
        else:
            path = self.path
        base, slug = op.split(path)
        self.settings.base = base
        self.settings.slug = slug

        self.site.pages.append(self)

    def __str__(self):
        return self.slug

    def get_relative_url(self):
        url = posixpath.join(self.base, self.slug)
        if self.isdir() and url and not url.endswith('/'):
            url += '/'
        return url

    @staticmethod
    def register(site):
        site.pages = []

events.connect('traverse-started', Page.register)


class Tag(Entry):

    prefix = 'tag' + op.sep

    @classmethod
    def check(cls, site, path):
        res = path.startswith(cls.prefix)
        return res

    def init(self):
        self.slug = self.path[len(self.prefix):-len('.html')]
        self.site.tag_cache[self.slug] = self

    def __str__(self):
        return self.slug

    def get_relative_url(self):
        if self.path.endswith('.html'):
            url = path2url(self.path[:-len('.html')])
        else:
            url = path2url(self.path)
        if self.isdir():
            url += '/'
        return url

    @staticmethod
    def register(site):
        site.tags = {}

    @classmethod
    def process(cls, site):
        site.tag_cache = {}
        for tag in site.tags:
            path = '%s%s.html' % (cls.prefix, tag)
            site.entries.append(cls(site, path, '_taglist.html'))

events.connect('traverse-started', Tag.register)
events.connect('site-traversed', Tag.process)


TYPE_LIST = [Post, Tag, Page, NonHTML]



import os, shutil, logging
import os.path as op
import sys

from cyrax.conf import Settings
from cyrax.template import initialize_env
from cyrax.utils import url2path, base_path
from cyrax.models import TYPE_LIST
from cyrax.events import events

try:
    import cyrax.rstpost
except ImportError:
    pass # no docutils :(


logger = logging.getLogger(__name__)


def ishidden(name):
    return name.startswith('.') or name.startswith('_')


def impcallback(relpath, root):
    if not root in sys.path:
        sys.path.insert(0, root)
    modname, cbname = relpath.rsplit('.', 1)
    mod = __import__(modname, {}, {}, [1])
    return getattr(mod, cbname)


def get_entry(site, path):
    try:
        Type = (t for t in TYPE_LIST if t.check(site, path)).next()
    except StopIteration:
        logger.error("Can't determine type for %s" % path)
        return
    return Type(site, path)


class Site(object):
    def __init__(self, root, dest):
        self.root = root
        if op.exists(dest):
            shutil.rmtree(dest)

        self.settings = Settings(parent_tmpl='_base.html')
        conf = op.join(self.root, 'settings.cfg')
        if op.exists(conf):
            self.settings.read(file(conf).read().decode('utf-8'))

        site_base_path = base_path(self.url)
        self.dest = op.join(dest, url2path(site_base_path[1:]))

        self.env = initialize_env(root)
        self.env.globals['site'] = self
        self.entries = []

        if self.settings.get('sitecallback'):
            callback = impcallback(self.settings.sitecallback, self.root)
            callback(self)

        self._traverse()

    @property
    def url(self):
        return getattr(self.settings, 'url', '/')

    def __repr__(self):
        return '<Site: %r>' % self.root

    def __getitem__(self, name):
        return self.settings[name]

    def __getattr__(self, name):
        try:
            return self.settings[name]
        except KeyError, e:
            raise AttributeError(str(e))

    def _traverse(self):
        events.emit('traverse-started', site=self)

        for path, _, files in os.walk(self.root):
            relative = path[len(self.root):].lstrip(os.sep)
            if (not relative.startswith('static') and
                not any(map(ishidden, relative.split(op.sep)))):
                for f in files:
                    if (f != 'settings.cfg' and
                        not ishidden(f) and
                        op.join(relative, f) not in self.settings.get('exclude',
                                                                      [])):
                        self.add_page(op.join(relative, f).replace('\\', '/'))

        events.emit('site-traversed', site=self)

    def add_page(self, path):
        self.entries.append(get_entry(self, path))

    def render(self):
        for entry in self.entries:
            entry.render()
        events.emit('site-rendered', site=self)
        self._copy_static()

    def _copy_static(self):
        if op.exists(op.join(self.root, 'static')):
            logger.info('Copying static files')
            shutil.copytree(op.join(self.root, 'static'),
                            op.join(self.dest, 'static'))



#!/usr/bin/env python

import os
from setuptools import setup

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

import cyrax

setup(name = 'cyrax',
      description = 'Static site generator',
      long_description = read('README'),
      license = 'BSD',
      version = cyrax.__version__,
      author = 'Alexander Solovyov',
      author_email = 'alexander@solovyov.net',
      url = 'http://hg.piranha.org.ua/cyrax/',
      install_requires = ['Jinja2', 'smartypants'],
      packages = ['cyrax', 'cyrax.template'],

      entry_points = {
        'console_scripts': ['cyrax = cyrax:main']
        },

      classifiers = [
        'Environment :: Console',
        'License :: OSI Approved :: BSD License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Utilities',
        ],
      platforms='any',
      )



from docutils import nodes, core
from docutils.writers import html4css1
from docutils.parsers.rst import directives, Directive

from cyrax import models
import cyrax.template.rstextensions


class CyraxWriter(html4css1.Writer):
    def __init__(self):
        html4css1.Writer.__init__(self)
        self.translator_class = CyraxTranslator
        self.visitor_attributes += ('cyraxmeta',)


class CyraxTranslator(html4css1.HTMLTranslator):
    def visit_cyraxmeta(self, node):
        self.cyraxmeta = '\n'.join(node.rawsource)

    def depart_cyraxmeta(self, node):
        pass


class CyraxMeta(Directive):
    '''Cyrax config catcher for reStructuredText

    Usage::

      .. meta::

        various: configuration
    '''
    class cyraxmeta(nodes.Special, nodes.PreBibliographic, nodes.Element):
        pass

    has_content = True

    def run(self):
        self.assert_has_content()
        return [self.cyraxmeta(self.content)]

        node = nodes.Element()
        node += nodes.raw(self.content)
        print self.content
        return [nodes.docinfo(self.content)]

directives.register_directive('meta', CyraxMeta)


class RstPost(models.Post):
    @staticmethod
    def check(site, path):
        return models.Post.check(site, path) and path.endswith('.rst')

    def init(self):
        # dumb hack
        self.settings.date = self.date
        self.site.posts.append(self)
        self.site.posts.sort(cmp=models.postcmp, reverse=True)
        self.site.latest_post = self.site.posts[0]

    def get_template(self):
        source = file(self.path).read()
        parts = core.publish_parts(source, writer=CyraxWriter())
        self.settings.read(parts['cyraxmeta'])
        return self.site.env.get_template(
            '_rstpost.html', globals={'entry': self, 'parts': parts})

    def __repr__(self):
        return '"%s"' % self.path


models.TYPE_LIST.insert(0, RstPost)



import urlparse
import os
import os.path as op
import posixpath
from itertools import izip, takewhile


def makedirs(path):
    try:
        os.makedirs(path)
    except OSError:
        pass


def safe_url_join(base, path):
    """
    >>> safe_url_join('blog/x', 'foo/bar')
    'blog/x/foo/bar'
    >>> safe_url_join('/blog/x/', 'foo/bar')
    '/blog/x/foo/bar'
    >>> safe_url_join('http://blog/x', 'foo/bar')
    'http://blog/x/foo/bar'
    """
    scheme, netloc, basepath, params, query, fragment = urlparse.urlparse(base)
    newpath = posixpath.join(basepath, path)
    parts = (scheme, netloc, newpath, params, query, fragment)
    return urlparse.urlunparse(parts)


def base_path(url):
    """
    >>> get_base_path('http://google.com')
    '/'
    >>> get_base_path('http://piranha.org.ua/blog')
    '/blog'
    """
    basepath = urlparse.urlparse(url)[2]
    return basepath or '/'


def url2path(url):
    return op.sep.join(url.split('/'))


def path2url(path):
    return '/'.join(path.split(op.sep))


def removecommon(p1, p2):
    common = takewhile(lambda x: x[0] == x[1], izip(p1, p2))
    l = len(list(common))
    return p1[l:], p2[l:]


def relpath(cur, dest):
    p1, p2 = removecommon(cur.split(op.sep), dest.split(op.sep))
    # empty element because of / in the end of name
    if p1 and p1[-1] == '':
        p1 = p1[:-1]
    p = ['../'] * len(p1) + p2
    if not p:
        return ''
    return op.join(*p)
